﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using ReservationProject.Data;
using ReservationProject.Models;
using ReservationProject.ModelsDto;
using ReservationProject.Services;

namespace ReservationProject.Controllers
{
    [Route("api/[controller]")]
    [ApiController]

    public class ReservationActions : ControllerBase
    {
        private readonly DataContext _context;
        private readonly IUserService _userService;

        public ReservationActions(DataContext dataContext, IUserService userService)
        {
            _context = dataContext;
            _userService = userService;
        }

        [HttpPost("add-reservation")]
        [Authorize]
        public IActionResult AddReservation(ReservationDTO reservation)
        {
            int userId = Convert.ToInt32( _userService.GetId());

            var rest = _context.Restaurants.SingleOrDefault(x => x.Id == reservation.Restaurant);

            var user = _context.Users.SingleOrDefault(x => x.Id == userId);

            if (rest == null)
                return BadRequest("There is no restaraunt");

            if (user == null)
                return BadRequest("There is no user");

            var newObject = new Reservation()
            {
                Id = 0,
                OperationDate = DateTime.SpecifyKind(DateTime.Now, DateTimeKind.Utc),
                ReservationDateTime = DateTime.SpecifyKind(reservation.ReservationDateTime, DateTimeKind.Utc),
                Restaurant = rest,
                User = user
            };

            _context.Reservations.Add(newObject);
            _context.SaveChanges();

            return Ok();
        }

        [HttpGet("get-reservations-by-user")]
        [Authorize]
        public IActionResult GetReservationsByUsers()
        {
            int userId = Convert.ToInt32(_userService.GetId());
            var user = _context.Users.SingleOrDefault(x => x.Id == userId);

            var reservations = _context.Reservations.Where(x => x.User.Id == userId)
                .Include(reservations => reservations.Restaurant).ToList();

            return Ok(reservations);
        }

    }
}
