﻿using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using ReservationProject.Data;
using ReservationProject.Models;
using ReservationProject.ModelsDto;

namespace ReservationProject.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class RestaurantActions : ControllerBase
    {
        private readonly DataContext _context;

        public RestaurantActions(DataContext context)
        {
            _context = context;
        }


        [HttpPost("add-restaurant")]
        public ActionResult AddRestaurant(RestaurantDTO restaurant)
        {
            var city = _context.Cities.SingleOrDefault( x => x.Name == restaurant.City );

            if (city == null)
                return BadRequest("There is no city in database");

            var newRestaurant = new Restaurant() {
                Id = 0,
                Name = restaurant.Name,
                Location = restaurant.Location,
                ContactNumber = restaurant.ContactNumber,
                City = city,    
                Photo = restaurant.Photo,
            };

            newRestaurant.City = city;

            _context.Restaurants.Add(newRestaurant);
            _context.SaveChanges();

            return Ok();
        }

        [HttpGet("list-restaurants")]
        public ActionResult ShowRestaurants()
        {          
            var restaurants = _context.Restaurants.
                Include(restaurants => restaurants.City).ToList();

            return Ok(restaurants);
        }

        [HttpGet("list-restaurants-by-city")]
        public ActionResult ShowRestaurantsByCity(string cityName)
        {
            var restaurants = _context.Restaurants.Include(
                restaurants => restaurants.City).Where( restaurants => 
                restaurants.City.Name == cityName).ToList();

            return Ok(restaurants);
        }

    }
}
