﻿using MailKit.Net.Smtp;
using MailKit.Security;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.IdentityModel.Tokens;
using MimeKit;
using ReservationProject.Data;
using ReservationProject.Models;
using ReservationProject.ModelsDto;
using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;

namespace ReservationProject.Controllers
{

    [Route("api/[controller]")]
    [ApiController]
    public class SignInActions : ControllerBase
    {
        private readonly DataContext _context;

        private readonly IConfiguration _configuration;


        public SignInActions(DataContext context, IConfiguration configuration)
        {
            _context = context;
            _configuration = configuration;
        }

        [HttpPost("registration")]
        public IActionResult Registration(UserDTO currentUser)
        {
            if (currentUser.Name.Length >= 20 || currentUser.Surname.Length >= 20)
                return BadRequest("Incorrect name or surname. The max value is reached");

                        
            var existingUser = _context.Users.SingleOrDefault(x => x.Email == currentUser.Email || x.PhoneNumber == currentUser.PhoneNumber);


            if (existingUser != null)
                return BadRequest("You already have an account");

            string code = CreateRandomToken();

            User newUser = new User
            {
                Id = 0,
                Name = currentUser.Name,
                Surname = currentUser.Surname,
                Email = currentUser.Email,
                PhoneNumber = currentUser.PhoneNumber,
                VerifyCode = code
            };


            _context.Users.Add(newUser);
            _context.SaveChangesAsync();

            return Ok("Successful registration");
        }

        [HttpPost("login")]
        public async Task<IActionResult> Login(string email)
        {
            var user = await _context.Users.FirstOrDefaultAsync(u => u.Email == email);

            if (user == null) return BadRequest("The user is not found");

            if (user.VerifyCode == null) return BadRequest("User is not verified");

            string token = CreateToken(user);

            return Ok(token);
        }

        [HttpPost("verify")]
        public async Task<IActionResult> Verify(string email, string emailCode) 
        {
            var user = await _context.Users.FirstOrDefaultAsync(_context => _context.Email == email);

            if (user == null) return BadRequest("Email is incorrect");

            if (emailCode != user.VerifyCode) return BadRequest("Sent code is incorrect");

            user.VerifiedAt = DateTime.SpecifyKind(DateTime.Now, DateTimeKind.Utc);
            AppContext.SetSwitch("Npgsql.EnableLegacyTimestampBehavior", true);
            await _context.SaveChangesAsync();

            return Ok("Thank you! Now you can use the application");
        }

        [HttpPost("send-code")]
        public async Task<IActionResult> SendEmailCode(string receiver)
        {
            string code = CreateRandomToken();

            var user = await _context.Users.FirstOrDefaultAsync(u => u.Email == receiver);

            if (user == null) 
                return NotFound("You need to complete registration");            

            user.VerifyCode = code;
            await _context.SaveChangesAsync();

            var email = new MimeMessage();
            email.From.Add(MailboxAddress.Parse("xisaby@gmail.com"));
            email.To.Add(MailboxAddress.Parse(receiver));
            email.Subject = "The Code for Reservation app";
            email.Body = new TextPart(MimeKit.Text.TextFormat.Html) { Text = "<h1>Hello, " + receiver + "!</h1>" + "<p>There are you`re code " + code + " </p>"  };

            using var smtp = new SmtpClient();
            smtp.Connect("smtp.gmail.com", 587, SecureSocketOptions.StartTls);
            smtp.Authenticate("xisaby@gmail.com", "enva menf bmhf xhjn");

            smtp.Send(email);
            smtp.Disconnect(true);

            return Ok();
        }

        private string CreateRandomToken()
        {
            return Convert.ToString(RandomNumberGenerator.GetInt32(1000, 9999));
        }

        private string CreateToken(User user)
        {
            List<Claim> claims = new List<Claim> {
                new Claim(ClaimTypes.Email, user.Email),
                new Claim("UserId" , user.Id.ToString()), // Костыль конечно но пойдет
            };

            var key = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(
                _configuration.GetSection("AppSettings:Token").Value!));

            var creds = new SigningCredentials(key, SecurityAlgorithms.HmacSha256Signature);

            var token = new JwtSecurityToken(
                claims: claims,
                expires: DateTime.Now.AddDays(7),
                signingCredentials: creds
                );

            var jwt = new JwtSecurityTokenHandler().WriteToken(token);

            return jwt;
        }
    }
}
